require_relative('../../puppet/parameter/netscaler_name')
require_relative('../../puppet/property/netscaler_truthy')

Puppet::Type.newtype(:netscaler_ntpserver) do
  @doc = 'Manage netscaler NTP server objects.'

  apply_to_device
  ensurable

  newparam(:name, :namevar => true) do #<String>
    desc "IP address or fully qualified domain name of the NTP server."
  end

  newproperty(:minimum_poll_interval) do #<Double>
    desc "Minimum time after which the NTP server must poll the NTP messages. In seconds, expressed as a power of 2.
Minimum value = 4
Maximum value = 17"
  end

  newproperty(:maximum_poll_interval) do #<Double>
    desc "Maximum time after which the NTP server must poll the NTP messages. In seconds, expressed as a power of 2.
Minimum value = 4
Maximum value = 17"
  end

  newproperty(:auto_key, :parent => Puppet::Property::NetscalerTruthy) do #<Boolean>
    truthy_property("Use the Autokey protocol for key management for this server, with the cryptographic values (for example, symmetric key, host and public certificate files, and sign key) generated by the ntp-keygen utility. To require authentication for communication with the server, you must set either the value of this parameter or the key parameter.",true,false)
  end

  newproperty(:key) do #<Double>
    desc "Key to use for encrypting authentication fields. All packets sent to and received from the server must include authentication fields encrypted by using this key. To require authentication for communication with the server, you must set either the value of this parameter or the autokey parameter.
Minimum value = 1
Maximum value = 65534"
  end

  newproperty(:preferred_ntp_server,:parent => Puppet::Property::NetscalerTruthy) do #<String>
    truthy_property("Preferred NTP server. The NetScaler appliance chooses this NTP server for time synchronization among a set of correctly operating hosts.
Default value: NO
Possible values = YES, NO","YES","NO")
  end
end
